//
//  Copyright (C) 2004-2007 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef MDFROOTOBJECT_H_
#define MDFROOTOBJECT_H_

#include "MdfModel.h"

BEGIN_NAMESPACE_MDFMODEL

    //-------------------------------------------------------------------------
    // DESCRIPTION:
    // Common base class for all MdfModel classes.  Provides support for
    // round-tripping unknown XML data from/to future schema versions.
    //-------------------------------------------------------------------------
    class MDFMODEL_API MdfRootObject
    {
    public:
        // Destruction
        virtual ~MdfRootObject() {}

        // Operations
        // Property : UnknownXml
        const MdfString& GetUnknownXml() const;
        void SetUnknownXml(const MdfString& strUnknownXml);

    protected:
        // Construction, initialization
        // Default constructor is protected to make this class abstract.
        MdfRootObject() {}

    private:
        // Data members
        // Cache for parser, keeping unknown XML for round-tripping.
        MdfString m_strUnknownXml;
    };

END_NAMESPACE_MDFMODEL
#endif //MDFROOTOBJECT_H_
